# -*- coding: utf-8 -*-
"""
Created on Mon Jan 03 15:37:53 2011 @author: tengen
Wasserbewegung in der Bodenzone - Isotopenaustausch (nach Criss) eines fliessenden und 
eines nich-fliessenden System
"""
from numpy import *
from fit_changed import *
import guiqwt.pyplot as plt
from math import log as ln

################################################################################
################################################################################
##Goodness of fit (Hornberger et al. 1992)
def good_fit(Cf,Cobs):
    Cmean=mean(Cobs)
    return 1-(sum((Cf-Cobs)**2)/sum((Cf-Cmean)**2))
    
##Delta Input aus Datei 
def c_in_p(monat):
    t=in_month-(monat+1)
    p=(in_month-(monat+1)>=0).nonzero()
    p= p[0]
    out=zeros((size(t)),float)
    out[0:size(p)]=in_dat_p[t[0:size(p)],0]
    out[size(p):size(t)]=mittel_in

    return out

##Gleichgewichtsfraktionierung Gas-Fluessigkeit: 2H, 18O (Majoube)
def majoube_l(T,isotop):   
    if isotop <> 'O' and isotop <> 'H':
        return 'Nur fuer 2H und 18O anwendbar'  
    if isotop=='O':
        a,b,c=1.137,-0.4156,-2.0667
    if isotop=='H':
        a,b,c=24.844,-76.248,52.612
        
##Saettigungsdampfdruck
def E(T):       #Annaeherung (besser als agnus, aber sehr aehnlich)
    if T==0:
        return 611.213
    T=T+273.15
    return (e**(-6094.4642/T+21.1249952-2.7245552*T/100.+1.6853396*(T**2)/100000.+2.4575506*ln(T)))/100.

##Gasmischung
def Rg(R1,R2,N1):
#    return R1*N1+R2*(1-N1)
    print R1*N1+R2*(1-N1)
    bruch=(R1*N1)/(1+R1)+(R2*(1-N1))/(1+R2)
    return bruch/(1-bruch)
    
##Isotopenverhältnis -2 buckets im geschlossenen System die sich mit Zeit annaehern nach Criss
def G(sa,sb,va,vb):
    return (sa*sb)/(sa+sb)*(va+vb)/(va*vb)
    
def Rw(Rw_a,Rw_b,k,sa,sb,va,vb,Tmp,t):
    Req=(va*Rw_a+vb*Rw_b)/(va+vb)
    return Req+exp(-k*G(sa,sb,va,vb)*E(Tmp)*t)*(Rw_a-Req)
#-------------------------------------------------------------------------------
##############
def c_out(z,params):
    
    v1,sa,sb,va,vb,shift=params
    v1=v1/12.
    k,Tmp=0.00647,12.
#    v1=120./12
#    sa,sb,va,vb=300.,300.,200.,200.
    z_max=-z[size(z)-1]     ##max Profiltiefe 
    cm=1.       ###Faktor um 'schichttiefe' einzustellen - 1-1cm; 5-5cm
    
    n_mon=(round((100+z_max/v1)/(exp(-k)))+1)        ##Anzahl der Isotopenmonate / vorlauf
    matrix=zeros((int(z_max/cm)+2,n_mon,2),float)   ##Berechnungmatrix / 5cm
    monat=arange(int(mon),int(mon+n_mon-1))         ##inputmonate
    
    matrix[0,1:,0]=c_in_p(monat)[::-1]+shift      #c_in_p(monat) 
    matrix[0,1:,1]=c_in_p(monat)[::-1]+shift      #c_in_p(monat) 
    matrix[1:,0,0]=mittel_in
    matrix[1:,0,1]=mittel_in


    tau=365./(v1*12.)*cm          ##verweilzeit pro cm in tagen

    for i in range(1,size(matrix[0,:,0])):            ##Zeit (monate n-inputwerte)  
        ##initialisierung der pro monat rechnung
        vmat=zeros((size(matrix[:,0,0]),1+int(round(v1/cm)),2),float)
        vmat[0,1:,0]=matrix[0,i,0]     
        vmat[0,1:,1]=matrix[0,i,1]    
        vmat[1:,0,0]=matrix[1:,i-1,0]     
        vmat[1:,0,1]=matrix[1:,i-1,1]
        
        for v in range(1,1+int(round(v1/cm))):           ##einzelne Bodnenschicht (5cm)
    
            ##langsames System
            vmat[1,v,0]=Rw(vmat[1,v-1,0],matrix[0,i,1],k,sa,sb,va,vb,Tmp,tau)
            vmat[2:,v,0]=Rw(vmat[2:,v-1,0],vmat[1:size(vmat[:0,0])-1,v-1,1],k,sa,sb,va,vb,Tmp,tau)
            ##schnelles system
            vmat[1,v,1]=Rw(matrix[0,i,1],vmat[1,v-1,0],k,sa,sb,va,vb,Tmp,tau)
            vmat[2:,v,1]=Rw(vmat[1:size(vmat[:0,0])-1,v-1,1],vmat[2:,v-1,0],k,sa,sb,va,vb,Tmp,tau)                     
    
        matrix[1:,i,0]=vmat[1:,v,0]                        ##langsames system
        matrix[1:,i,1]=vmat[1:,v,1]                        ##schnelles system
    
    
    s_a=matrix[1:,size(matrix[0,:,0])-1,0]
    s_b=matrix[1:,size(matrix[0,:,0])-1,1]
    
    z_ind=zeros((size(z)),int)
    z_ind[:]=-z[:]/cm
    
    result=zeros((size(z)+1,5),float)
    result[1:,0]=z
    result[1:,1]=arange(cm/2,size(matrix[:,0,0])*cm,cm)[z_ind]
    result[1:,2]=s_a[z_ind]
    result[1:,3]=s_b[z_ind]
    result[1:,4]=(s_a[z_ind]*va+s_b[z_ind]*vb)/(va+vb)

    return result[1:,4]

#################################################################################
def equi_fit(z,y,t_t):
        
    v1 = FitParam("Sickergeschwidigkeit (cm/a)", 120., 5., 1000.)
    sa = FitParam("Oberflaeche_lW (cm*cm)", 300., 1., 1000.)
    sb = FitParam("Oberflaeche_fW (cm*cm)", 300., 1., 1000.)
    va = FitParam("Volumen_lW (cm*cm*cm)", 200., 1., 1000.)
    vb = FitParam("Volumen_fW (cm*cm*cm)", 200., 1., 1000.)
    shift = FitParam("Versatz (1/1000)", 0., -3., 3.)  
    params = [v1,sa,sb,va,vb,shift]

    values = guifit(z, y, c_out, params, xlabel="Tiefe (cm)", ylabel="Isotop")

    return values
#    
#################################################################################
#
##Input
in_n_18o='O18_N_alpha_Freiburg.txt'
'''Daten Beginn:15.06.98, Ende:15.07.10, Monatswerte (15....)'''

#in_n_18o='test\in_data\BSc_GruppeP7.txt'
'''Daten Beginn:15.06.98, Ende:15.12.11, Monatswerte (15....)'''


in_data,mon='jakob.txt',11.
#in_data,mon='test\in_data\kaiserstuhl_1.txt',12.
#in_data,mon='test\in_data\O18_Strassbourg_P7.txt',2.

#in_month=size(in_dat_p[:,0])
#soil_w_monat=in_dat_p[:,1]*in_dat_p[:,2]
#mittel_in=sum(in_dat_p[:,0]*soil_w_monat)
#===============================================================================
#plt.plot(in_dat_p[in_month:in_month,0],'r-')
#plt.plot(in_dat_p[:,0],'r-')
#plt.plot(soil_w_monat,in_dat_p[:,0],'b+')
#plt.show()

if __name__ == '__main__':
    
    in_dat_p=loadtxt(in_n_18o)      ##O,N,alpha niederschlag
    in_dat=loadtxt(in_data)         ##profil
    
    in_month=size(in_dat_p[:,0])
    soil_w_monat=in_dat_p[:,1]*in_dat_p[:,2]
    mittel_in=sum(in_dat_p[:,0]*soil_w_monat)
    
    z,y=-in_dat[:,0],in_dat[:,1]
    
    SHOW = True    
    t_t=mon/12.
    
    a=equi_fit(-z,y,t_t)
    values=a
     ##root-mean-square error /normalized
    model=c_out(-z,values)
    rmse=(sum((model-y)**2)/size(y))**0.5
    n_rmse=rmse/(max(y)-min(y))
    
    print 'Sickergeschwindigkeit:',values[0],'cm pro Jahr'
    print 'Oberflaeche lW:', values[1],'[cm²]'
    print 'Oberflaeche fW:',values[2], '[cm²]'
    print 'Volumen lW:', values[3],'[cm³]'
    print 'Volumen fW:',values[4], '[cm³]'
    print 'Qualitaet der Modellierung:'
    print 'Root Mean Square Error:', rmse
    print 'Normalized Root Mean Square Error:',n_rmse
    print 'Goodness of fit:',good_fit(model,y)
    
    plt.plot(y,z,'b--')
    plt.plot(model,z,'r-')
    plt.show()
    plt.ylabel('Depth [cm]')
    plt.xlabel('Delta [1/1000]')
    print 'end'

################################################################################





