# -*- coding: utf-8 -*-
"""
Created on Mon Nov 28 15:37:53 2011 @author: tengen
Wasserbewegung in der Bodenzone
"""
from numpy import *
from fit_changed import *
import guiqwt.pyplot as plt 

################################################################################

################################################################################
##Goodness of fit (Hornberger et al. 1992)
def good_fit(Cf,Cobs):
    Cmean=mean(Cobs)
    return 1-(sum((Cf-Cobs)**2)/sum((Cf-Cmean)**2))
    
##Delta Input aus Datei 
def c_in_p(monat):
    t=in_month-(monat+1)
    if monat > size(in_dat_p[:,0]):
        return mittel_in, mean(soil_w_monat)
    else:
        return in_dat_p[t,0], soil_w_monat[t]

def p(z,v,d):        ##Dispersionsparameter (z:cm, v:cm/a, d:cm²/a, p:-)
    return d/(v*z)

def MT(z,v):        ##Mittlere Transitzeit für die Distanz x mit v
    return z/v  

def c_out(z,params):        ##Summation    
    v,d,v_f,shift=params        
    
    v=v/12.*e**(z/v_f)
    d=d/12.
    if d/v[0] < 1:
        i_fac=50.
    else:
        i_fac=10.*d/v[0] 
    
    p_d=p(z,v,d) 
    M=MT(z,v)
    
    t=int(t_t*12)

    sum_n=size(in_dat_p[:,0])-(t+1)
    profil_dpth=z[size(z)-1]
    if profil_dpth/v[0] > sum_n:
        sum_n=int(profil_dpth)+36
    
    tau_mat=arange(1./i_fac,sum_n+1,1./i_fac,float)
  
    tau_mat=tau_mat[0:sum_n*int(i_fac)]
    in_cp_mat=zeros((sum_n*int(i_fac),2),float)
         
    for i in range(0,sum_n+1):
        in_cp_mat[i*i_fac:(i*i_fac)+i_fac,0]=c_in_p(i+t)[0]+shift        
        in_cp_mat[i*i_fac:(i*i_fac)+i_fac,1]=c_in_p(i+t)[1]     
    
    in_n_weight=in_cp_mat[:,1]/sum(in_cp_mat[:,1])    
    dep_d_mat=zeros(size(z),float)

    for dep in range(0,size(z)):
        weight_dep=in_n_weight*(1./(tau_mat*(4.*pi*p_d[dep]*tau_mat/M[dep])**0.5)*e**(-(1-tau_mat/M[dep])**2/(4.*p_d[dep]*tau_mat/M[dep])))/sum(in_n_weight*(1./(tau_mat*(4.*pi*p_d[dep]*tau_mat/M[dep])**0.5)*e**(-(1-tau_mat/M[dep])**2/(4.*p_d[dep]*tau_mat/M[dep]))))
        dep_d_mat[dep]=nansum((in_cp_mat[:,0])*weight_dep)
#        dep_d_mat[dep]=nansum((1./i_fac)*(in_cp_mat)*(1./(tau_mat*(4.*pi*p_d[dep]*tau_mat/M[dep])**0.5)*e**(-(1-tau_mat/M[dep])**2/(4.*p_d[dep]*tau_mat/M[dep]))))
    
    return dep_d_mat

################################################################################
def disp_fit(z,y,t_t):
        
    v = FitParam("Sickergeschwidigkeit (cm/a)", 100., 1., 500.)
    d = FitParam("Dispersionsparameter (cm2/a)", 5., 1., 500.)
    v_f = FitParam("Geschwindigkeitsdaempfung", 500., 50., 1000.) 
    shift = FitParam("Versatz (1/1000)", 0., -3., 3.) 
    
    params = [v,d,v_f,shift]

    values = guifit(z, y, c_out, params, xlabel="Tiefe (cm)", ylabel="Isotop")

    return values
    
################################################################################
################################################################################
##Input
in_n_18o='O18_N_alpha_Freiburg.txt'
'''Daten Beginn:15.06.98, Ende:15.07.10, Monatswerte (15....)'''

#in_n_18o='test\in_data\BSc_GruppeP7.txt'
#'''Daten Beginn:15.06.98, Ende:15.12.11, Monatswerte (15....)'''
#in_n_18o='test\in_data\BSc_GruppeP6.txt'


in_data,mon='jakob.txt',11.
#in_data,mon='test\in_data\kaiserstuhl_1.txt',12.
#in_data,mon='test\in_data\O18_Strassbourg_P7.txt',2.
#in_data,mon='test\in_data\D_Hausen_moehlin_P6.txt',2.
#in_data,mon='test\in_data\D_Hausen_probe_muddy_place.txt',0

#===============================================================================
#plt.plot(in_dat_p[in_month:in_month,0],'r-')
#plt.plot(in_dat_p[:,0],'r-')
#plt.plot(soil_w_monat,in_dat_p[:,0],'b+')
#plt.show()


if __name__ == '__main__':
    
    in_dat_p=loadtxt(in_n_18o)      ##O,N,alpha niederschlag
    in_dat=loadtxt(in_data)         ##profil
    
    in_month=size(in_dat_p[:,0])
    soil_w_monat=in_dat_p[:,1]*in_dat_p[:,2]
    mittel_in=sum(in_dat_p[:,0]*soil_w_monat)
    
    z,y=in_dat[:,0],in_dat[:,1]
    
    SHOW = True    
    t_t=mon/12.
    
    a=disp_fit(-z,y,t_t)
    values=a
    
     ##root-mean-square error /normalized
    model=c_out(-z,values)
    rmse=(sum((model-y)**2)/size(y))**0.5
    n_rmse=rmse/(max(y)-min(y))
    
    print 'Sickergeschwindigkeit:',values[0],'cm pro Jahr'
    print 'Dispersion:', values[1],'[cm2/a]'
    print 'Geschwindigkeitsdaempfung:',values[2]
    print 'Versatz:',values[3], '[1/1000]'
    print 'Qualitaet der Modellierung:'
    print 'Root Mean Square Error:', rmse
    print 'Normalized Root Mean Square Error:',n_rmse
    print 'Goodness of fit:',good_fit(model,y)
    
    plt.plot(y,z,'b--',label='Data')
    plt.plot(c_out(-z,values),z,'r-',label='Model')
    plt.ylabel('Depth [cm]')
    plt.xlabel('Delta [1/1000]')
    plt.legend()
    plt.show()
    
    print 'end'




